'' +--------------------------------------------------------------------------+
'' | Cluso's RamBlade & TriBladeProp:  PropBootSD (boot from SD card)   v1.10 |
'' +--------------------------------------------------------------------------+
'' |  Author:            "Cluso99" (Ray Rodrick)                              |
'' |  Copyright (c) 2010 "Cluso99" (Ray Rodrick)                              |
'' |  License            MIT License - See end of file for terms of use       |
'' +--------------------------------------------------------------------------+
                             
'' Thanks to Jeff Ledger for PropDOS, Mike Green & Tomas Rokicki for modified Fsrw code (from FemtoBASIC)

'' This is a minimal eeprom boot code to automatically boot a Propeller Binary
''
'' RR20100108   PropBootSD_V1_00  minimal boot version (fixed DO tristate when booting)
'' RR20120120   PropBootSD_V1_10  mod for RamBlade3
'' RR20120814   Strip to simplify as an example for Prop II

CON

'RamBlade3
  _clkmode  = xtal1 + pll16x
  _xinfreq  = 6_500_000
  spiDO     = 20                'microSD
  spiClk    = 21
  spiDI     = 22
  spiCS     = 23
  SIpin     = 31                'serial
  SOpin     = 30
  baud      = 115200

  offset    = $180              'offset on MBR where ident and pointer will be written
      
OBJ
     fsrw  : "fsrwFemto_rr005a"         'modified SD routines
     uart  : "FullDuplexSerial_rr004"
      
VAR
     long   ioControl[2]
     byte   buff[512]
          
PUB mainProgram | r, fbase, fsize

  uart.start(SIpin,SOpin,0,baud)
  waitcnt(clkfreq * 3 + cnt)
  uart.str(string(13,10))                               'crlf
  uart.str(string("RamBlade3 SD test loader for P2 V1.12"))
  uart.str(string(13,10))                               'crlf

'***** locate the file *****
  r := fsrw.start(@ioControl)                           'start I2C/SPI driver
  r := fsrw.mount(spiDO,spiClk,spiDI,spiCS)             'check for SD card, abort if not found.
  if r < 0
    uart.str(string("** SD card not found **"))
    failure                                             'failure routine

  DisplayMBR                                            'display the MBR

{{--- the following may be disabled ---}}
  WriteMBR(fbase)                                       'locate file & update MBR to point to sector# for booting
{{--- ----------------------------- ---}}

  fbase := buff[offset+15] << 24 | buff[offset+14] << 16 | buff[offset+13] << 8 | buff[offset+12] 'get the sector# for booting
  fsize := 32768                                                                ' force the size
    
  uart.str(string("Booting file... at sector = "))
  uart.dec(fbase)
  uart.str(string(" $"))
  uart.hex(fbase,8)
  uart.str(string(", size = "))
  uart.dec(fsize)
  uart.str(string(13,10))                               'crlf
  waitcnt(clkfreq / 2 + cnt)                              
  uart.stop
' fsrw.bootSDCard                                       'boots the prop using the sector# in datablock and the size of the filesize
  fsrw.sdboot(fbase, fsize)                             'boot te prop passing sector# and size

' ** here on failure **
  uart.str(string("** Boot failed **"))
  failure                                               'failure routine

PRI WriteMBR (fbase) | r, fsize

  'locate the file and determine the sector# for the first data record
  r := fsrw.popen(string("_VER    .CMD"),"r")           'open the prop binary boot file (locates file and size)
  if r <> 0
    uart.str(string("** file not found **"))
    failure                                             'failure routine

  fbase := fsrw.datablock                               'get the files first data sector
  fsize := fsrw.fsize                                   'get the files data size
  uart.str(string("File located... at sector = "))
  uart.dec(fbase)
  uart.str(string(", size = "))
  uart.dec(fsize)
  uart.str(string(13,10))                               'crlf

  'read the MBR sector
  r := fsrw.readSDCard(0, @buff, 512)                   'read the MBR sector

  'update the MBR sector
  buff[offset+00] := "P"                                '\\ setup 12 byte identification string at $1E0
  buff[offset+01] := "r"                                '||   "PropellerII" +$00
  buff[offset+02] := "o"                                '||
  buff[offset+03] := "p"                                '||
  buff[offset+04] := "e"                                '||
  buff[offset+05] := "l"                                '||
  buff[offset+06] := "l"                                '||
  buff[offset+07] := "e"                                '||
  buff[offset+08] := "r"                                '||
  buff[offset+09] := "I"                                '||
  buff[offset+10] := "I"                                '||
  buff[offset+11] := 0                                  '//
  buff[offset+12] := (fbase ) & $FF                     '\\ setup the sector# where the file data resides
  buff[offset+13] := (fbase >> 8) & $FF                 '||
  buff[offset+14] := (fbase >> 16) & $FF                '||   (check endian is correct !!!!!!!!!!!)
  buff[offset+15] := (fbase >> 24) & $FF                '//
 
  'writeback the MBR sector
  r := fsrw.writeSDCard(0, @buff, 512)                  'write back the MBR sector

  DisplayMBR                                            'display the updated MBR sector

Pri DisplayMBR | r, i, j
  r := fsrw.readSDCard(0, @buff, 512)                   'read the MBR sector
  uart.str(string("MBR sector 0..."))
  uart.tx(13)                                           'cr
  repeat j from 0 to 31
    repeat i from 0 to 15
      uart.hex(buff[i + j*16],2)
      uart.tx(" ")
    uart.tx(" ")
    repeat i from 0 to 15
      if buff[i+ j*16] < 31
        uart.tx(".")
      else
        uart.tx(buff[i + j*16])
    uart.str(string(13))                                'crlf


Pri Failure | t
'***** here only if it fails *****
  uart.str(string(13,10))                               'crlf
  waitcnt(clkfreq * 5 + cnt)                            
  uart.stop
  repeat t from 0 to 7                                  'Stop all COGs except this one
    if (t <> cogid)
      cogstop(t)
  cogstop(cogid)                                        'then stop this cog. All done!


dat                                                   
{{
+------------------------------------------------------------------------------------------------------------------------------+
|                                                   TERMS OF USE: MIT License                                                  |                                                            
+------------------------------------------------------------------------------------------------------------------------------+
|Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    | 
|files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    |
|modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software|
|is furnished to do so, subject to the following conditions:                                                                   |
|                                                                                                                              |
|The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.|
|                                                                                                                              |
|THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          |
|WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         |
|COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   |
|ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         |
+------------------------------------------------------------------------------------------------------------------------------+
}} 